#include "em_device.h"
#include "em_chip.h"

//#include "hal-config.h"
#include <stdint.h>
#include <stdbool.h>
//#include "efm32.h"
//#include "efm32_chip.h"
#include "em_cmu.h"
#include "em_gpio.h"
#include "em_usart.h"

#define autoCS 0
#define PIN_CS  14
#define PORT_CS gpioPortC
static uint8_t spis_idx = 0;

void SPI_Initial(bool master)
{
	CMU_ClockEnable(cmuClock_HFPER, true);
	CMU_ClockEnable(cmuClock_GPIO, true);
	CMU_ClockEnable(cmuClock_USART1, true);

	USART_InitSync_TypeDef SPI_init = USART_INITSYNC_DEFAULT;
//	SPI_init.baudrate = 1000000;
	SPI_init.autoCsEnable = autoCS;
	SPI_init.msbf = true;
	SPI_init.master = master;

	USART_InitSync(USART1, &SPI_init);
	if (master)
	{
		GPIO_PinModeSet(gpioPortD, 7, gpioModePushPull, 0); //MO
		GPIO_PinModeSet(gpioPortD, 6, gpioModeInput, 0); //MI

		GPIO_PinModeSet(gpioPortC, 15, gpioModePushPull, 0); //clk
		GPIO_PinModeSet(PORT_CS, PIN_CS, gpioModePushPull, 1); //cs
		GPIO_DriveModeSet(PORT_CS,gpioDriveModeHigh);
	}
	else
	{
		GPIO_PinModeSet(gpioPortD, 7, gpioModeInput, 0); //MO
		GPIO_PinModeSet(gpioPortD, 6, gpioModePushPull, 1); //MI

		GPIO_PinModeSet(gpioPortC, 15, gpioModeInput, 0); //clk
		GPIO_PinModeSet(PORT_CS, PIN_CS, gpioModeInput, 1); //cs

	}

#if autoCS
	USART1->ROUTE |= USART_ROUTE_CSPEN;
#endif
	if(!master)
		USART1->ROUTE |= USART_ROUTE_CSPEN;

	USART1->ROUTE |=  USART_ROUTE_CLKPEN | USART_ROUTE_TXPEN
			| USART_ROUTE_RXPEN | USART_ROUTE_LOCATION_LOC3;
}

void delay(uint32_t count)
{
	while (count--)
		;
}

void CSBegin(void)
{
#if !autoCS
	GPIO_PinOutClear(PORT_CS, PIN_CS);
	delay(40);
#endif
}

void CSEnd(void)
{
#if !autoCS
	delay(40);
	GPIO_PinOutSet(PORT_CS, PIN_CS);
#endif
}

void test1by1(void)
{
	for (int i = 0; i < 256; i++)
	{
		CSBegin();
		USART_SpiTransfer(USART1, i & 0xff);
		CSEnd();
	}
}

void test256(void)
{
	CSBegin();
	for (int i = 0; i < 256; i++)
	{
		USART_SpiTransfer(USART1, i & 0xff);
	}
	CSEnd();
}

void teststr(void)
{
	char *pStr = "efm32zgM";

	CSBegin();
	for (int i = 0; i < strlen(pStr); i++)
	{
//		USART_SpiTransfer(USART1, *(pStr + i));
		USART_Tx(USART1, *(pStr + i));
	}
	CSEnd();
}

void spiM(void)
{
	SPI_Initial(true);
	/* Infinite loop */
	while (1)
	{
//		test1by1();
		teststr();
		delay(10000);
	}
}


void spiS(void)
{

	char *pStr = "efm32zgS901234";
	uint8_t len = strlen(pStr);

	SPI_Initial(false);
	while(1)
	{
		uint8_t tx = 0xAA;
		if(spis_idx < len)
			tx = *(pStr + spis_idx++);
		else
			spis_idx = 0;

		 USART_Tx(USART1, tx);

//		 if(USART1->IF & USART_IF_TXUF)
//		 {
//			 spis_idx = 0;
//		 }
	}
}

int main(void)
{
	/* Chip errata */
	CHIP_Init();
	SystemCoreClockUpdate();
//	spiS();
	spiM();

}



